import { Calendar } from '@fullcalendar/core';
import { OPTION_IS_COMPLEX } from './options';
import { shallowCopy, mapHash } from './utils';
import { wrapVDomGenerator, createVueContentTypePlugin } from './custom-content-type';
import { defineComponent, getCurrentInstance, onBeforeUnmount, onBeforeUpdate, onMounted, ref, watch } from 'vue';
export default defineComponent({
    name: 'FullCalendar',
    props: {
        options: {
            type: Object,
            default: () => ({})
        }
    },
    render(createElement) {
        return createElement('div', {
            // when renderId is changed, Vue will trigger a real-DOM async rerender, calling beforeUpdate/updated
            attrs: { 'data-fc-render-id': this.renderId }
        });
    },
    setup(props, { expose }) {
        const renderId = ref(0);
        const instance = getCurrentInstance();
        const root = instance && instance.proxy;
        if (!root) {
            return {};
        }
        function buildOptions(suppliedOptions) {
            let internal = root && root.$options;
            suppliedOptions = suppliedOptions || {};
            return {
                ...internal.scopedSlotOptions,
                ...suppliedOptions,
                plugins: (suppliedOptions.plugins || []).concat([
                    createVueContentTypePlugin(root)
                ])
            };
        }
        function getApi() {
            return (root && root.$options).calendar;
        }
        onMounted(() => {
            let internal = root.$options;
            internal.scopedSlotOptions = mapHash(root.$scopedSlots, wrapVDomGenerator); // needed for buildOptions
            const calendar = new Calendar(root.$el, buildOptions(props.options));
            internal.calendar = calendar;
            calendar.render();
        });
        onBeforeUpdate(() => getApi().resumeRendering());
        onBeforeUnmount(() => getApi().destroy());
        watch(() => props.options, val => {
            let calendar = getApi();
            calendar.pauseRendering();
            calendar.resetOptions(buildOptions(val));
            renderId.value++; // will queue a rerender
        }, { deep: true });
        for (const complexOptionName in OPTION_IS_COMPLEX) {
            watch(() => props.options[complexOptionName], val => {
                // unfortunately the handler is called with undefined if new props were set, but the complex one wasn't ever set
                if (val !== undefined) {
                    let calendar = getApi();
                    calendar.pauseRendering();
                    calendar.resetOptions({
                        [complexOptionName]: shallowCopy(val)
                    }, true);
                    renderId.value++;
                }
            });
        }
        const exposed = {
            getApi,
            renderId
        };
        expose(exposed);
        return exposed;
    }
});
//# sourceMappingURL=FullCalendar.js.map